@@ -30,11 +30,11 @@
  *
  * When xchat calls python code:
  *   - Change the current_plugin for the executing plugin;
- *   - Acquire the global interpreter lock
  *   - Release xchat lock
+ *   - Acquire the global interpreter lock
  *   - Make the python call
- *   - Acquire xchat lock
  *   - Release the global interpreter lock
+ *   - Acquire xchat lock
  *
  * When python code calls xchat:
  *   - Release the global interpreter lock
@@ -65,45 +65,58 @@
 
 #define VERSION_MAJOR 0
 #define VERSION_MINOR 1
-#define VERSION "0.1"
+#define VERSION "0.2"
 
 #ifdef WIN32
 #undef WITH_THREAD /* Thread support locks up xchat on Win32. */
 #endif
 
+#define NONE 0
+#define ALLOW_THREADS 1
+#define RESTORE_CONTEXT 2
+
 #ifdef WITH_THREAD
 #define ACQUIRE_XCHAT_LOCK() PyThread_acquire_lock(xchat_lock, 1)
 #define RELEASE_XCHAT_LOCK() PyThread_release_lock(xchat_lock)
-#define BEGIN_XCHAT_CALLS() \
-	Py_BEGIN_ALLOW_THREADS \
-	ACQUIRE_XCHAT_LOCK(); \
-	Py_END_ALLOW_THREADS
+#define BEGIN_XCHAT_CALLS(x) \
+	do { \
+		PyObject *calls_plugin = NULL; \
+		PyThreadState *calls_thread; \
+		if ((x) & RESTORE_CONTEXT) \
+			calls_plugin = Plugin_GetCurrent(); \
+		calls_thread = PyEval_SaveThread(); \
+		ACQUIRE_XCHAT_LOCK(); \
+		if (!((x) & ALLOW_THREADS)) { \
+			PyEval_RestoreThread(calls_thread); \
+			calls_thread = NULL; \
+		} \
+		if (calls_plugin) \
+			xchat_set_context(ph, \
+				Plugin_GetContext(calls_plugin)); \
+		while (0)
 #define END_XCHAT_CALLS() \
-	RELEASE_XCHAT_LOCK()
+		RELEASE_XCHAT_LOCK(); \
+		if (calls_thread) \
+			PyEval_RestoreThread(calls_thread); \
+	} while(0)
 #else
 #define ACQUIRE_XCHAT_LOCK()
 #define RELEASE_XCHAT_LOCK()
-#define BEGIN_XCHAT_CALLS()
+#define BEGIN_XCHAT_CALLS(x)
 #define END_XCHAT_CALLS()
 #endif
 
-#define RESTORE_CONTEXT() \
-	do { \
-		PyObject *plugin = Plugin_GetCurrent(); \
-		if (plugin) \
-			xchat_set_context(ph, Plugin_GetContext(plugin)); \
-	} while (0)
-
 #define BEGIN_PLUGIN(plg) \
 	do { \
-	Plugin_AcquireThread(plg); \
-	Plugin_SetContext(plg, xchat_get_context(ph)); \
+	xchat_context *begin_plugin_ctx = xchat_get_context(ph); \
 	RELEASE_XCHAT_LOCK(); \
+	Plugin_AcquireThread(plg); \
+	Plugin_SetContext(plg, begin_plugin_ctx); \
 	} while (0)
 #define END_PLUGIN(plg) \
 	do { \
-	ACQUIRE_XCHAT_LOCK(); \
 	Plugin_ReleaseThread(plg); \
+	ACQUIRE_XCHAT_LOCK(); \
 	} while (0)
 
 #define Plugin_Swap(x) \
@@ -597,8 +610,11 @@ XChatOut_write(PyObject *self, PyObject *args)
 	char *data, *pos;
 	if (!PyArg_ParseTuple(args, "s#:write", &data, &data_size))
 		return NULL;
-	BEGIN_XCHAT_CALLS();
-	RESTORE_CONTEXT();
+	if (!data_size) {
+		Py_INCREF(Py_None);
+		return Py_None;
+	}
+	BEGIN_XCHAT_CALLS(RESTORE_CONTEXT|ALLOW_THREADS);
 	if (((XChatOutObject *)self)->softspace) {
 		add_space = 1;
 		((XChatOutObject *)self)->softspace = 0;
@@ -623,15 +639,14 @@ XChatOut_write(PyObject *self, PyObject *args)
 			/* Return something valid, since we have
 			 * already warned the user, and he probably
 			 * won't be able to notice this exception. */
-			Py_INCREF(Py_None);
-			return Py_None;
+			goto exit;
 		}
 		xchatout_buffer = new_buffer;
 	}
 	memcpy(xchatout_buffer+xchatout_buffer_pos, data, data_size);
 	print_limit = new_buffer_pos = xchatout_buffer_pos+data_size;
 	pos = xchatout_buffer+print_limit;
-	if (add_space) {
+	if (add_space && *(pos-1) != '\n') {
 		*pos = ' ';
 		*(pos+1) = 0;
 		new_buffer_pos++;
@@ -656,6 +671,8 @@ XChatOut_write(PyObject *self, PyObject *args)
 	} else {
 		xchatout_buffer_pos = new_buffer_pos;
 	}
+
+exit:
 	END_XCHAT_CALLS();
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -742,7 +759,7 @@ Context_command(ContextObject *self, PyObject *args)
 	char *text;
 	if (!PyArg_ParseTuple(args, "s:command", &text))
 		return NULL;
-	BEGIN_XCHAT_CALLS();
+	BEGIN_XCHAT_CALLS(ALLOW_THREADS);
 	xchat_set_context(ph, self->context);
 	xchat_command(ph, text);
 	END_XCHAT_CALLS();
@@ -756,7 +773,7 @@ Context_prnt(ContextObject *self, PyObject *args)
 	char *text;
 	if (!PyArg_ParseTuple(args, "s:prnt", &text))
 		return NULL;
-	BEGIN_XCHAT_CALLS();
+	BEGIN_XCHAT_CALLS(ALLOW_THREADS);
 	xchat_set_context(ph, self->context);
 	xchat_print(ph, text);
 	END_XCHAT_CALLS();
@@ -764,6 +781,27 @@ Context_prnt(ContextObject *self, PyObject *args)
 	return Py_None;
 }
 
+static PyObject *
+Context_emit_print(ContextObject *self, PyObject *args)
+{
+	char *argv[10];
+	char *name;
+	int res;
+	memset(&argv, 0, sizeof(char*)*10);
+	if (!PyArg_ParseTuple(args, "s|ssssss:print_event", &name,
+			      &argv[0], &argv[1], &argv[2],
+			      &argv[3], &argv[4], &argv[5],
+			      &argv[6], &argv[7], &argv[8]))
+		return NULL;
+	BEGIN_XCHAT_CALLS(ALLOW_THREADS);
+	xchat_set_context(ph, self->context);
+	res = xchat_emit_print(ph, name, argv[0], argv[1], argv[2],
+					 argv[3], argv[4], argv[5],
+					 argv[6], argv[7], argv[8]);
+	END_XCHAT_CALLS();
+	return PyInt_FromLong(res);
+}
+
 static PyObject *
 Context_get_info(ContextObject *self, PyObject *args)
 {
@@ -771,7 +809,7 @@ Context_get_info(ContextObject *self, PyObject *args)
 	char *name;
 	if (!PyArg_ParseTuple(args, "s:get_info", &name))
 		return NULL;
-	BEGIN_XCHAT_CALLS();
+	BEGIN_XCHAT_CALLS(NONE);
 	xchat_set_context(ph, self->context);
 	info = xchat_get_info(ph, name);
 	END_XCHAT_CALLS();
@@ -798,6 +836,7 @@ static PyMethodDef Context_methods[] = {
 	{"set", (PyCFunction) Context_set, METH_NOARGS},
 	{"command", (PyCFunction) Context_command, METH_VARARGS},
 	{"prnt", (PyCFunction) Context_prnt, METH_VARARGS},
+	{"emit_print", (PyCFunction) Context_emit_print, METH_VARARGS},
 	{"get_info", (PyCFunction) Context_get_info, METH_VARARGS},
 	{"get_list", (PyCFunction) Context_get_list, METH_VARARGS},
 	{NULL, NULL}
@@ -861,7 +900,7 @@ Context_FromServerAndChannel(char *server, char *channel)
 {
 	ContextObject *ctxobj;
 	xchat_context *context;
-	BEGIN_XCHAT_CALLS();
+	BEGIN_XCHAT_CALLS(NONE);
 	context = xchat_find_context(ph, server, channel);
 	END_XCHAT_CALLS();
 	if (context == NULL)
@@ -991,6 +1030,7 @@ Plugin_New(char *filename, PyMethodDef *xchat_methods, PyObject *xcoobj)
 {
 	PluginObject *plugin = NULL;
 	PyObject *m, *o;
+	char *argv[] = {"<xchat>", 0};
 
 	if (filename) {
 		char *old_filename = filename;
@@ -1023,6 +1063,7 @@ Plugin_New(char *filename, PyMethodDef *xchat_methods, PyObject *xcoobj)
 		goto error;
 	}
 
+	PySys_SetArgv(1, argv);
 	PySys_SetObject("__plugin__", (PyObject *) plugin);
 
 	/* Set stdout and stderr to xchatout. */
@@ -1178,7 +1219,7 @@ Plugin_RemoveHook(PyObject *plugin, Hook *hook)
 		/* Ok, unhook it. */
 		if (hook->type == HOOK_XCHAT) {
 			/* This is an xchat hook. Unregister it. */
-			BEGIN_XCHAT_CALLS();
+			BEGIN_XCHAT_CALLS(NONE);
 			xchat_unhook(ph, (xchat_hook*)hook->data);
 			END_XCHAT_CALLS();
 		}
@@ -1199,7 +1240,7 @@ Plugin_RemoveAllHooks(PyObject *plugin)
 		Hook *hook = (Hook *) list->data;
 		if (hook->type == HOOK_XCHAT) {
 			/* This is an xchat hook. Unregister it. */
-			BEGIN_XCHAT_CALLS();
+			BEGIN_XCHAT_CALLS(NONE);
 			xchat_unhook(ph, (xchat_hook*)hook->data);
 			END_XCHAT_CALLS();
 		}
@@ -1222,7 +1263,12 @@ Plugin_Delete(PyObject *plugin)
 			PyObject *retobj;
 			retobj = PyObject_CallFunction(hook->callback, "(O)",
 						       hook->userdata);
-			Py_DECREF(retobj);
+			if (retobj) {
+				Py_DECREF(retobj);
+			} else {
+				PyErr_Print();
+				PyErr_Clear();
+			}
 		}
 		list = list->next;
 	}
@@ -1296,8 +1342,7 @@ Module_xchat_command(PyObject *self, PyObject *args)
 	char *text;
 	if (!PyArg_ParseTuple(args, "s:command", &text))
 		return NULL;
-	BEGIN_XCHAT_CALLS();
-	RESTORE_CONTEXT();
+	BEGIN_XCHAT_CALLS(RESTORE_CONTEXT|ALLOW_THREADS);
 	xchat_command(ph, text);
 	END_XCHAT_CALLS();
 	Py_INCREF(Py_None);
@@ -1310,14 +1355,33 @@ Module_xchat_prnt(PyObject *self, PyObject *args)
 	char *text;
 	if (!PyArg_ParseTuple(args, "s:prnt", &text))
 		return NULL;
-	BEGIN_XCHAT_CALLS();
-	RESTORE_CONTEXT();
+	BEGIN_XCHAT_CALLS(RESTORE_CONTEXT|ALLOW_THREADS);
 	xchat_print(ph, text);
 	END_XCHAT_CALLS();
 	Py_INCREF(Py_None);
 	return Py_None;
 }
 
+static PyObject *
+Module_xchat_emit_print(PyObject *self, PyObject *args)
+{
+	char *argv[10];
+	char *name;
+	int res;
+	memset(&argv, 0, sizeof(char*)*10);
+	if (!PyArg_ParseTuple(args, "s|ssssss:print_event", &name,
+			      &argv[0], &argv[1], &argv[2],
+			      &argv[3], &argv[4], &argv[5],
+			      &argv[6], &argv[7], &argv[8]))
+		return NULL;
+	BEGIN_XCHAT_CALLS(RESTORE_CONTEXT|ALLOW_THREADS);
+	res = xchat_emit_print(ph, name, argv[0], argv[1], argv[2],
+					 argv[3], argv[4], argv[5],
+					 argv[6], argv[7], argv[8]);
+	END_XCHAT_CALLS();
+	return PyInt_FromLong(res);
+}
+
 static PyObject *
 Module_xchat_get_info(PyObject *self, PyObject *args)
 {
@@ -1325,8 +1389,7 @@ Module_xchat_get_info(PyObject *self, PyObject *args)
 	char *name;
 	if (!PyArg_ParseTuple(args, "s:get_info", &name))
 		return NULL;
-	BEGIN_XCHAT_CALLS();
-	RESTORE_CONTEXT();
+	BEGIN_XCHAT_CALLS(RESTORE_CONTEXT);
 	info = xchat_get_info(ph, name);
 	END_XCHAT_CALLS();
 	if (info == NULL) {
@@ -1336,6 +1399,40 @@ Module_xchat_get_info(PyObject *self, PyObject *args)
 	return PyString_FromString(info);
 }
 
+static PyObject *
+Module_xchat_get_prefs(PyObject *self, PyObject *args)
+{
+	PyObject *res;
+	const void *info;
+	char *name;
+	int type;
+	if (!PyArg_ParseTuple(args, "s:get_prefs", &name))
+		return NULL;
+	BEGIN_XCHAT_CALLS(NONE);
+	type = xchat_get_prefs(ph, name, (const char**)&info, (int*)&info);
+	END_XCHAT_CALLS();
+	switch (type) {
+		case 0:
+			Py_INCREF(Py_None);
+			res = Py_None;
+			break;
+		case 1:
+			res = PyString_FromString((char*)info);
+			break;
+		case 2:
+		case 3:
+			res = PyInt_FromLong((int)info);
+			break;
+		default:
+			PyErr_Format(PyExc_RuntimeError,
+				     "unknown get_prefs type (%d), "
+				     "please report", type);
+			res = NULL;
+			break;
+	}
+	return res;
+}
+
 static PyObject *
 Module_xchat_get_context(PyObject *self, PyObject *args)
 {
@@ -1400,7 +1497,7 @@ Module_xchat_hook_command(PyObject *self, PyObject *args, PyObject *kwargs)
 	if (hook == NULL)
 		return NULL;
 
-	BEGIN_XCHAT_CALLS();
+	BEGIN_XCHAT_CALLS(NONE);
 	hook->data = (void*)xchat_hook_command(ph, name, priority,
 					       Callback_Command, help, hook);
 	END_XCHAT_CALLS();
@@ -1436,7 +1533,7 @@ Module_xchat_hook_server(PyObject *self, PyObject *args, PyObject *kwargs)
 	if (hook == NULL)
 		return NULL;
 
-	BEGIN_XCHAT_CALLS();
+	BEGIN_XCHAT_CALLS(NONE);
 	hook->data = (void*)xchat_hook_server(ph, name, priority,
 					      Callback_Command, hook);
 	END_XCHAT_CALLS();
@@ -1472,7 +1569,7 @@ Module_xchat_hook_print(PyObject *self, PyObject *args, PyObject *kwargs)
 	if (hook == NULL)
 		return NULL;
 
-	BEGIN_XCHAT_CALLS();
+	BEGIN_XCHAT_CALLS(NONE);
 	hook->data = (void*)xchat_hook_print(ph, name, priority,
 					     Callback_Print, hook);
 	END_XCHAT_CALLS();
@@ -1508,7 +1605,7 @@ Module_xchat_hook_timer(PyObject *self, PyObject *args, PyObject *kwargs)
 	if (hook == NULL)
 		return NULL;
 
-	BEGIN_XCHAT_CALLS();
+	BEGIN_XCHAT_CALLS(NONE);
 	hook->data = (void*)xchat_hook_timer(ph, timeout,
 					     Callback_Timer, hook);
 	END_XCHAT_CALLS();
@@ -1587,8 +1684,7 @@ Module_xchat_get_list(PyObject *self, PyObject *args)
 	l = PyList_New(0);
 	if (l == NULL)
 		return NULL;
-	BEGIN_XCHAT_CALLS();
-	RESTORE_CONTEXT();
+	BEGIN_XCHAT_CALLS(RESTORE_CONTEXT);
 	list = xchat_list_get(ph, (char*)name);
 	if (list == NULL)
 		goto error;
@@ -1629,14 +1725,16 @@ Module_xchat_get_list(PyObject *self, PyObject *args)
 		}
 	}
 	xchat_list_free(ph, list);
-	END_XCHAT_CALLS();
-	return l;
+	goto exit;
 error:
 	if (list)
 		xchat_list_free(ph, list);
 	Py_DECREF(l);
+	l = NULL;
+
+exit:
 	END_XCHAT_CALLS();
-	return NULL;
+	return l;
 }
 
 static PyObject *
@@ -1677,8 +1775,12 @@ static PyMethodDef Module_xchat_methods[] = {
 		METH_VARARGS},
 	{"prnt",		Module_xchat_prnt,
 		METH_VARARGS},
+	{"emit_print",		Module_xchat_emit_print,
+		METH_VARARGS},
 	{"get_info",		Module_xchat_get_info,
 		METH_VARARGS},
+	{"get_prefs",		Module_xchat_get_prefs,
+		METH_VARARGS},
 	{"get_context",		Module_xchat_get_context,
 		METH_NOARGS},
 	{"find_context",	(PyCFunction)Module_xchat_find_context,
@@ -1754,6 +1856,7 @@ IInterp_Cmd(char *word[], char *word_eol[], void *userdata)
 	char *channel = (char *) xchat_get_info(ph, "channel");
 	g_return_val_if_fail(channel != NULL, 0);
 	if (channel[0] == '>' && strcmp(channel, ">>python<<") == 0) {
+		xchat_printf(ph, ">>> %s\n", word_eol[1]);
 		IInterp_Exec(word_eol[1]);
 		return 1;
 	}
@@ -1934,6 +2037,7 @@ xchat_plugin_init(xchat_plugin *plugin_handle,
 	*plugin_desc = "Python scripting interface";
 
 	/* Initialize python. */
+	Py_SetProgramName("xchat");
 	Py_Initialize();
 	PySys_SetArgv(1, argv);
 
