@@ -15,7 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
-#define VERSION "1.0.42"
+#define VERSION "1.0.48"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -60,9 +60,26 @@ static int nextprocid = 0x1000;
 #define PROCPREFIX "__xctcl_"
 
 static char unknown[] = {
-    "proc ::unknown {args} {\n"
-        "global errorInfo errorCode\n"
-        "set cmd [lindex $args 0]\n" "if { [string index $cmd 0] == \"/\" } {\n" "command \"[string range $cmd 1 end] [join [lrange $args 1 end] \" \"]\"\n" "} else {\n" "return -code error \"invalid command name \\\"$cmd\\\"\"\n" "}\n" "}"
+"rename unknown iunknown\n"
+"proc ::unknown {args} {\n"
+"  global errorInfo errorCode\n"
+"  if { [string index [lindex $args 0] 0] == \"/\" } {\n"
+"    command \"[string range [join $args \" \"] 1 end]\"\n"
+"  } else {\n"
+"    set code [catch {uplevel iunknown $args} msg]\n"
+"    if {$code == 1} {\n"
+"      set new [split $errorInfo \\n]\n"
+"      set new [join [lrange $new 0 [expr {[llength $new] - 8}]] \\n]\n"
+"      return -code error -errorcode $errorCode -errorinfo $new $msg\n"
+"    } else {\n"
+"      return -code $code $msg\n"
+"    }\n"
+"  }\n"
+"}\n"
+"proc unsupported0 {from to {bytes \"\"}} {\n"
+"  set b [expr {$bytes == \"\" ? \"\" : \"-size [list $bytes]\"}]\n"
+"  eval [list fcopy $from $to] $b\n"
+"}\n"
 };
 
 static char sourcedirs[] = {
@@ -75,7 +92,6 @@ static char sourcedirs[] = {
 };
 
 static char inlinetcl[] = {
-
 "proc splitsrc { } {\n"
 "uplevel \"scan \\$_src \\\"%\\\\\\[^!\\\\\\]!%\\\\\\[^@\\\\\\]@%s\\\" _nick _ident _host\"\n"
 "}\n"
@@ -274,7 +290,7 @@ static void queue_nexttimer()
     time_t then;
 
     nexttimerindex = 0;
-    then = (time_t) 9999999999;
+    then = (time_t) 0x7fffffff;
 
     for (x = 1; x < MAX_TIMERS; x++) {
         if (timers[x].timerid) {
@@ -286,7 +302,7 @@ static void queue_nexttimer()
     }
 }
 
-static int insert_timer(int seconds, char *script)
+static int insert_timer(int seconds, int count, char *script)
 {
     int x;
     int dummy;
@@ -310,6 +326,8 @@ static int insert_timer(int seconds, char *script)
             }
             timers[x].timerid = (nexttimerid++ % INT_MAX) + 1;
             timers[x].timestamp = now + seconds;
+            timers[x].count = count;
+            timers[x].seconds = seconds;
             timers[x].procPtr = StrDup(InternalProcName(id), &dummy);
             queue_nexttimer();
             return (timers[x].timerid);
@@ -342,12 +360,19 @@ static void do_timer()
     }
     xchat_set_context(ph, origctx);
 
-    timers[index].timerid = 0;
-    if (timers[index].procPtr != NULL) {
-        DeleteInternalProc(timers[index].procPtr);
-        Tcl_Free(timers[index].procPtr);
+    if (timers[index].count != -1)
+      timers[index].count--;
+
+    if (timers[index].count == 0) {
+      timers[index].timerid = 0;
+      if (timers[index].procPtr != NULL) {
+          DeleteInternalProc(timers[index].procPtr);
+          Tcl_Free(timers[index].procPtr);
+      }
+      timers[index].procPtr = NULL;
+    } else {
+      timers[index].timestamp += timers[index].seconds;
     }
-    timers[index].procPtr = NULL;
 
     queue_nexttimer();
 
@@ -639,6 +664,8 @@ static int tcl_timers(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
             Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].timerid));
             Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].timestamp - now));
             Tcl_DStringAppendElement(&ds, timers[x].procPtr);
+	    Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].seconds));
+	    Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].count));
             Tcl_DStringEndSublist(&ds);
         }
     }
@@ -653,15 +680,35 @@ static int tcl_timer(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     int seconds;
     int timerid;
+    int repeat = 0;
+    int count = 0;
+    int first = 1;
     char reply[32];
 
-    BADARGS(3, 3, " seconds {script | procname ?args?}");
+    BADARGS(3, 6, " ?-repeat? ?-count times? seconds {script | procname ?args?}");
 
-    if (Tcl_GetInt(irp, argv[1], &seconds) != TCL_OK) {
-        return TCL_ERROR;
+    while (argc--) {
+        if (strcasecmp(argv[first], "-repeat") == 0) {
+            repeat++;
+        } else if (strcasecmp(argv[first], "-count") == 0) {
+            if (Tcl_GetInt(irp, argv[++first], &count) != TCL_OK)
+                return TCL_ERROR;
+        } else {
+            break;
+        }
+        first++;
     }
 
-    if ((timerid = insert_timer(seconds, argv[2])) == -1) {
+    if (repeat && !count)
+      count = -1;
+
+    if (!count)
+      count = 1;
+
+    if (Tcl_GetInt(irp, argv[first++], &seconds) != TCL_OK)
+        return TCL_ERROR;
+
+    if ((timerid = insert_timer(seconds, count, argv[first++])) == -1) {
         Tcl_AppendResult(irp, "0", NULL);
         return TCL_ERROR;
     }
@@ -987,6 +1034,28 @@ static int tcl_getinfo(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     return TCL_OK;
 }
 
+static int tcl_prefs(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
+{
+    int i;
+    const char *str;
+
+    BADARGS(2, 2, " name");
+
+    switch (xchat_get_prefs (ph, argv[1], &str, &i)) {
+        case 1:
+            Tcl_AppendResult(irp, str, NULL);
+            break;
+        case 2:
+        case 3:
+            Tcl_AppendResult(irp, myitoa(i), NULL);
+            break;
+        default:
+            Tcl_AppendResult(irp, NULL);
+    }
+
+    return TCL_OK;
+}
+
 static int tcl_server(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
@@ -1963,12 +2032,14 @@ static void Tcl_Plugin_Init()
     Tcl_CreateCommand(interp, "off", tcl_off, NULL, NULL);
     Tcl_CreateCommand(interp, "nickcmp", tcl_xchat_nickcmp, NULL, NULL);
     Tcl_CreateCommand(interp, "print", tcl_print, NULL, NULL);
+    Tcl_CreateCommand(interp, "prefs", tcl_prefs, NULL, NULL);
     Tcl_CreateCommand(interp, "::puts", tcl_xchat_puts, NULL, NULL);
     Tcl_CreateCommand(interp, "queries", tcl_queries, NULL, NULL);
     Tcl_CreateCommand(interp, "raw", tcl_raw, NULL, NULL);
     Tcl_CreateCommand(interp, "server", tcl_server, NULL, NULL);
     Tcl_CreateCommand(interp, "servers", tcl_servers, NULL, NULL);
     Tcl_CreateCommand(interp, "setcontext", tcl_setcontext, NULL, NULL);
+    Tcl_CreateCommand(interp, "tab", tcl_channel, NULL, NULL);
     Tcl_CreateCommand(interp, "timer", tcl_timer, NULL, NULL);
     Tcl_CreateCommand(interp, "timerexists", tcl_timerexists, NULL, NULL);
     Tcl_CreateCommand(interp, "timers", tcl_timers, NULL, NULL);
@@ -1988,12 +2059,12 @@ static void Tcl_Plugin_Init()
     for (x = 0; x < XC_SIZE; x++)
         xc[x].hook = NULL;
 
+    xchatdir = xchat_get_info(ph, "xchatdir");
+
     if (Tcl_Eval(interp, unknown) == TCL_ERROR) {
         xchat_printf(ph, "Error sourcing internal 'unknown' (%s)\n", Tcl_GetStringResult(interp));
     }
 
-    xchatdir = xchat_get_info(ph, "xchatdir");
-
     if (Tcl_Eval(interp, sourcedirs) == TCL_ERROR) {
         xchat_printf(ph, "Error sourcing internal 'sourcedirs' (%s)\n", Tcl_GetStringResult(interp));
     }
@@ -2001,7 +2072,6 @@ static void Tcl_Plugin_Init()
     if (Tcl_Eval(interp, inlinetcl) == TCL_ERROR) {
         xchat_printf(ph, "Error sourcing internal 'inlinetcl' (%s)\n", Tcl_GetStringResult(interp));
     }
-
 }
 
 static void Tcl_Plugin_DeInit()
