@@ -1,13 +1,15 @@
 /* cfgfiles.h */
 
-extern char *xdir;
+extern char *xdir_fs;
+extern char *xdir_utf;
 
 char *cfg_get_str (char *cfg, char *var, char *dest);
 int cfg_get_bool (char *var);
 int cfg_get_int_with_result (char *cfg, char *var, int *result);
 int cfg_get_int (char *cfg, char *var);
 int cfg_put_int (int fh, int value, char *var);
-char *get_xdir (void);
+char *get_xdir_fs (void);
+char *get_xdir_utf8 (void);
 void load_config (void);
 int save_config (void);
 void list_free (GSList ** list);
@@ -15,6 +17,7 @@ void list_loadconf (char *file, GSList ** list, char *defaultconf);
 int list_delentry (GSList ** list, char *name);
 void list_addentry (GSList ** list, char *cmd, char *name);
 int cmd_set (session *sess, char *tbuf, char *word[], char *word_eol[]);
+int mkdir_utf8 (char *dir);
 
 #define STRUCT_OFFSET_STR(type,field) \
 ( (unsigned int) (((char *) (&(((type *) NULL)->field)))- ((char *) NULL)) )
