@@ -56,7 +56,7 @@ void fe_url_add (const char *text);
 void fe_pluginlist_update (void);
 void fe_buttons_update (struct session *sess);
 void fe_dlgbuttons_update (struct session *sess);
-void fe_dcc_send_filereq (struct session *sess, char *nick, int maxcps);
+void fe_dcc_send_filereq (struct session *sess, char *nick, int maxcps, int passive);
 void fe_set_channel (struct session *sess);
 void fe_set_title (struct session *sess);
 void fe_set_nonchannel (struct session *sess, int state);
