@@ -9,5 +9,5 @@ struct history
 
 void history_add (struct history *his, char *text);
 void history_free (struct history *his);
-char *history_up (struct history *his);
+char *history_up (struct history *his, char *current_text);
 char *history_down (struct history *his);
