@@ -231,7 +231,8 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 {
 	session *def = sess;
 	server *serv = sess->server;
-	int beep = 0;
+	int beep = FALSE;
+	int hilight = FALSE;
 
 	if (!fromme)
 	{
@@ -269,10 +270,14 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 
 	if (!fromme)
 	{
+		hilight = is_hilight (text, sess, serv);
+		if (hilight && prefs.beephilight)
+			beep = TRUE;
+
 		if (beep || sess->beep)
 			fe_beep ();
 
-		if (is_hilight (text, sess, serv))
+		if (hilight)
 		{
 			EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, NULL, NULL, 0);
 			return;
