@@ -129,6 +129,9 @@ struct xchatprefs
 	char proxy_host[64];
 	int proxy_port;
 	int proxy_type; /* 0=disabled, 1=wingate 2=socks4, 3=socks5, 4=http */
+	unsigned int proxy_auth;
+	char proxy_user[32];
+	char proxy_pass[32];
 
 	int first_dcc_send_port;
 	int last_dcc_send_port;
