@@ -168,9 +168,9 @@ fe_args (int argc, char *argv[])
 	{
 		if (!strcasecmp (argv[1+offset], "-d") || !strcasecmp (argv[1+offset], "--cfgdir"))
 		{
-			xdir = strdup (argv[2+offset]);
-			if (xdir[strlen (xdir) - 1] == '/')
-				xdir[strlen (xdir) - 1] = 0;
+			xdir_fs = strdup (argv[2+offset]);
+			if (xdir_fs[strlen (xdir_fs) - 1] == '/')
+				xdir_fs[strlen (xdir_fs) - 1] = 0;
 			offset += 2;
 		}
 	}
@@ -190,7 +190,7 @@ fe_args (int argc, char *argv[])
 	return 1;
 }
 
-static const char rc[] =
+const char cursor_color_rc[] =
 	"style \"xc-ib-st\""
 	"{"
 		"GtkEntry::cursor-color=\"#%02x%02x%02x\""
@@ -220,8 +220,8 @@ create_input_style (void)
 	if (prefs.style_inputbox && !done_rc)
 	{
 		done_rc = TRUE;
-		sprintf (buf, rc, (colors[18].red >> 8), (colors[18].green >> 8),
-					(colors[18].blue >> 8));
+		sprintf (buf, cursor_color_rc, (colors[18].red >> 8),
+			(colors[18].green >> 8), (colors[18].blue >> 8));
 		gtk_rc_parse_string (buf);
 	}
 
@@ -688,7 +688,7 @@ fe_set_throttle (server *serv)
 		sess = list->data;
 		if (sess->server == serv)
 		{
-			snprintf (tbuf, sizeof (tbuf) - 1, "%d bytes", serv->sendq_len);
+			snprintf (tbuf, sizeof (tbuf) - 1, _("%d bytes"), serv->sendq_len);
 
 			if (!sess->gui->is_tab || current_tab == sess)
 			{
