@@ -1,4 +1,3 @@
-#define GTK_DISABLE_DEPRECATED
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -61,6 +60,7 @@ enum
 	ST_RADIO,
 	ST_NUMBER,
 	ST_HSCALE,
+	ST_FRAME,
 	ST_LABEL
 };
 
@@ -246,9 +246,15 @@ static const setting network_settings[] =
 {
 	{ST_ENTRY,	N_("Address to bind to:"), P_OFFSET(hostname), 0, 0, sizeof prefs.hostname},
 	{ST_LABEL,	N_("(Only useful for computers with multiple addresses).")},
-	{ST_ENTRY,	N_("Proxy server:"), P_OFFSET(proxy_host), 0, 0, sizeof prefs.proxy_host},
-	{ST_NUMBER,	N_("Proxy port:"), P_OFFINT(proxy_port), 0, 0, 65535},
-	{ST_MENU,	N_("Proxy type:"), P_OFFINT(proxy_type), 0, proxytypes, 0},
+	{ST_FRAME,	N_("Proxy server"), 0, 0, 0, 0},
+	{ST_ENTRY,	N_("Hostname:"), P_OFFSET(proxy_host), 0, 0, sizeof prefs.proxy_host},
+	{ST_ENTRY,	N_("Username:"), P_OFFSET(proxy_user), 0, 0, sizeof prefs.proxy_user},
+	{ST_ENTRY,	N_("Password:"), P_OFFSET(proxy_pass), 0, GINT_TO_POINTER(1), sizeof prefs.proxy_pass},
+	{ST_NUMBER,	N_("Port:"), P_OFFINT(proxy_port), 0, 0, 65535},
+	{ST_MENU,	N_("Type:"), P_OFFINT(proxy_type), 0, proxytypes, 0},
+
+	{ST_TOGGLE,	N_("Authenticate to the proxy server (only HTTP)"), P_OFFINT(proxy_auth), 0, 0, 0},
+
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
@@ -322,11 +328,25 @@ setup_create_spin (GtkWidget *table, int row, const setting *set)
 	}
 }
 
+static gint
+setup_apply_tint (int *tag)
+{
+	prefs.tint_red = setup_prefs.tint_red;
+	prefs.tint_green = setup_prefs.tint_green;
+	prefs.tint_blue = setup_prefs.tint_blue;
+	mg_update_xtext (current_sess->gui->xtext);
+	*tag = 0;
+	return 0;
+}
 
 static void
 setup_hscale_cb (GtkHScale *wid, const setting *set)
 {
+	static int tag = 0;
+
 	setup_set_int (&setup_prefs, set, gtk_range_get_value(GTK_RANGE(wid)));
+	if(tag == 0)
+		tag = g_idle_add ((GSourceFunc)setup_apply_tint, &tag);
 }
 
 static void
@@ -379,11 +399,24 @@ setup_create_radio (GtkWidget *table, int row, setting *set)
 }
 #endif
 
+static GtkWidget *proxy_user; 	/* username GtkEntry */
+static GtkWidget *proxy_pass; 	/* password GtkEntry */
+
+
 static void
 setup_menu_cb (GtkWidget *item, const setting *set)
 {
-	setup_set_int (&setup_prefs, set,
-						GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "n")));
+	int n = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "n"));
+
+	/* set the prefs.<field> */
+	setup_set_int (&setup_prefs, set, n);
+
+	if (g_object_get_data (G_OBJECT (item), "p"))
+	{
+		/* only HTTP can use a username/pass */
+		gtk_widget_set_sensitive (proxy_user, (n == 4));
+		gtk_widget_set_sensitive (proxy_pass, (n == 4));
+	}
 }
 
 static void
@@ -408,6 +441,11 @@ setup_create_menu (GtkWidget *table, int row, const setting *set)
 	{
 		item = gtk_menu_item_new_with_label (_(text[i]));
 		g_object_set_data (G_OBJECT (item), "n", GINT_TO_POINTER (i));
+
+		/* set a flag for the callback to use */
+		if (text == proxytypes)
+			g_object_set_data (G_OBJECT (item), "p", GINT_TO_POINTER (1));	
+
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		g_signal_connect (G_OBJECT (item), "activate",
@@ -508,6 +546,8 @@ setup_create_entry (GtkWidget *table, int row, const setting *set)
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 
 	wid = gtk_entry_new ();
+	if (set->list)
+		gtk_entry_set_visibility (GTK_ENTRY (wid), FALSE);
 	if (set->tooltip)
 		add_tip (wid, _(set->tooltip));
 	gtk_entry_set_max_length (GTK_ENTRY (wid), set->extra);
@@ -515,6 +555,17 @@ setup_create_entry (GtkWidget *table, int row, const setting *set)
 	g_signal_connect (G_OBJECT (wid), "changed",
 							G_CALLBACK (setup_entry_cb), (gpointer)set);
 
+	if (set->offset == P_OFFSET(proxy_user))
+		proxy_user = wid;
+	if (set->offset == P_OFFSET(proxy_pass))
+		proxy_pass = wid; 
+
+	/* only http can auth */
+	if ( (set->offset == P_OFFSET(proxy_pass) ||
+			set->offset == P_OFFSET(proxy_user)) &&
+	     setup_prefs.proxy_type != 4)
+		gtk_widget_set_sensitive (wid, FALSE);
+
 	if (set->type == ST_ENTRY)
 		gtk_table_attach (GTK_TABLE (table), wid, 1, 5, row, row + 1,
 								GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
@@ -535,34 +586,56 @@ setup_create_entry (GtkWidget *table, int row, const setting *set)
 }
 
 static GtkWidget *
-setup_create_page (const setting *set)
+setup_create_frame (char *label, GtkWidget **left, GtkWidget **right, GtkWidget *box)
 {
-	int i, row;
-	GtkWidget *tab, *box, *hbox, *left, *right;
+	GtkWidget *tab, *hbox, *frame, *inbox = box;
 
-	box = gtk_vbox_new (FALSE, 20);
-	gtk_container_set_border_width (GTK_CONTAINER (box), 4);
+	if (label)
+	{
+		frame = gtk_frame_new (label);
+		inbox = gtk_vbox_new (FALSE, 3);
+		gtk_container_set_border_width (GTK_CONTAINER (inbox), 5);
+		gtk_container_add (GTK_CONTAINER (frame), inbox);
+		gtk_container_add (GTK_CONTAINER (box), frame);
+	}
 
 	tab = gtk_table_new (3, 2, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (tab), 2);
 	gtk_table_set_row_spacings (GTK_TABLE (tab), 2);
 	gtk_table_set_col_spacings (GTK_TABLE (tab), 3);
-	gtk_container_add (GTK_CONTAINER (box), tab);
+	gtk_container_add (GTK_CONTAINER (inbox), tab);
 
 	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (box), hbox);
+	gtk_container_add (GTK_CONTAINER (inbox), hbox);
 
-	left = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (hbox), left);
+	*left = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (hbox), *left);
 
-	right = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (hbox), right);
+	*right = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (hbox), *right);
+
+	return tab;
+}
+
+static GtkWidget *
+setup_create_page (const setting *set)
+{
+	int i, row;
+	GtkWidget *tab, *box, *left, *right;
+
+	box = gtk_vbox_new (FALSE, 20);
+	gtk_container_set_border_width (GTK_CONTAINER (box), 4);
+
+	tab = setup_create_frame (NULL, &left, &right, box);
 
 	i = row = 0;
 	while (set[i].type != ST_END)
 	{
 		switch (set[i].type)
 		{
+		case ST_FRAME:
+			tab = setup_create_frame (_(set[i].label), &left, &right, box);
+			break;
 		case ST_EFONT:
 		case ST_ENTRY:
 		case ST_EFILE:
@@ -671,7 +744,8 @@ setup_create_color_button (GtkWidget *table, int num, int row, int col)
 	else
 		sprintf (buf, "%d", num);
 	but = gtk_button_new_with_label (buf);
-	g_object_set_data (G_OBJECT (but), "xchat-color", 0);
+	/* win32 build uses this to turn off themeing */
+	g_object_set_data (G_OBJECT (but), "xchat-color", (gpointer)1);
 	gtk_table_attach (GTK_TABLE (table), but, col, col+1, row, row+1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 	g_signal_connect (G_OBJECT (but), "clicked",
@@ -909,6 +983,14 @@ setup_apply_to_sess (session_gui *gui)
 
 	if (prefs.style_inputbox)
 	{
+		extern char cursor_color_rc[];
+		char buf[256];
+		sprintf (buf, cursor_color_rc,
+				(colors[18].red >> 8),
+				(colors[18].green >> 8),
+				(colors[18].blue >> 8));
+		gtk_rc_parse_string (buf);
+
 		setup_apply_entry_style (gui->input_box);
 		setup_apply_entry_style (gui->limit_entry);
 		setup_apply_entry_style (gui->key_entry);
@@ -951,13 +1033,8 @@ setup_apply (struct xchatprefs *pr)
 
 	memcpy (&prefs, pr, sizeof (prefs));
 
-#ifdef WIN32
-	mkdir (prefs.dccdir);
-	mkdir (prefs.dcc_completed_dir);
-#else
-	mkdir (prefs.dccdir, S_IRUSR | S_IWUSR | S_IXUSR);
-	mkdir (prefs.dcc_completed_dir, S_IRUSR | S_IWUSR | S_IXUSR);
-#endif
+	mkdir_utf8 (prefs.dccdir);
+	mkdir_utf8 (prefs.dcc_completed_dir);
 
 	if (new_pix)
 	{
